@@ -26,14 +26,13 @@ Code to reproduce the benchmark results:
 import copy
 import os
 
+import time
 import lightly
 import numpy as np
 import pytorch_lightning as pl
 import torch
 import torch.nn as nn
-import torch.nn.functional as F
 import torchvision
-from lightly.models.modules import NNMemoryBankModule
 from lightly.models.modules.heads import BYOLProjectionHead
 from lightly.models.modules.heads import MoCoProjectionHead
 from lightly.models.modules.heads import ProjectionHead
@@ -45,8 +44,6 @@ from lightly.models.utils import deactivate_requires_grad
 from lightly.models.utils import update_momentum
 from lightly.utils import BenchmarkModule
 from pytorch_lightning.loggers import TensorBoardLogger
-from torchvision import transforms
-from torchvision.transforms.transforms import CenterCrop
 
 num_workers = 8
 memory_bank_size = 4096
@@ -59,13 +56,35 @@ knn_k = 200
 knn_t = 0.1
 classes = 10
 
+# Set to True to enable Distributed Data Parallel training.
+distributed = False
+
+# Set to True to enable Synchronized Batch Norm (requires distributed=True). 
+# If enabled the batch norm is calculated over all gpus, otherwise the batch
+# norm is only calculated from samples on the same gpu.
+sync_batchnorm = False
+
+# Set to True to gather features from all gpus before calculating 
+# the loss (requires distributed=True).
+# If enabled then the loss on every gpu is calculated with features from all 
+# gpus, otherwise only features from the same gpu are used.
+gather_distributed = False 
+
 # benchmark
 n_runs = 1 # optional, increase to create multiple runs and report mean + std
 batch_sizes = [128, 512]
 
 # use a GPU if available
-gpus = -1 if torch.cuda.is_available() else 0
-distributed_backend = 'ddp' if torch.cuda.device_count() > 1 else None
+gpus = torch.cuda.device_count() if torch.cuda.is_available() else 0
+
+if distributed:
+    distributed_backend = 'ddp'
+    # reduce batch size for distributed training
+    batch_sizes = [size // gpus for size in batch_sizes]
+else:
+    distributed_backend = None
+    # limit to single gpu if not using distributed training
+    gpus = min(gpus, 1)
 
 # Adapted from our MoCo Tutorial on CIFAR-10
 #
@@ -103,6 +122,7 @@ collate_fn = lightly.data.SimCLRCollateFunction(
 swav_collate_fn = lightly.data.SwaVCollateFunction(
     crop_sizes=[32],
     crop_counts=[2], # 2 crops @ 32x32px
+    crop_min_scales=[0.14]
 )
 
 # No additional augmentations for the test set
@@ -162,13 +182,13 @@ def get_data_loaders(batch_size: int, multi_crops: bool = False):
 
     return dataloader_train_ssl, dataloader_train_kNN, dataloader_test
 
-
 class MocoModel(BenchmarkModule):
     def __init__(self, dataloader_kNN, num_classes):
         super().__init__(dataloader_kNN, num_classes)
 
         # create a ResNet backbone and remove the classification head
-        resnet = lightly.models.ResNetGenerator('resnet-18', num_splits=8)
+        num_splits = 0 if sync_batchnorm else 8
+        resnet = lightly.models.ResNetGenerator('resnet-18', num_splits=num_splits)
         self.backbone = nn.Sequential(
             *list(resnet.children())[:-1],
             nn.AdaptiveAvgPool2d(1)
@@ -198,13 +218,13 @@ class MocoModel(BenchmarkModule):
         update_momentum(self.projection_head, self.projection_head_momentum, 0.99)
 
         def step(x0_, x1_):
-            x1_, shuffle = batch_shuffle(x1_)
+            x1_, shuffle = batch_shuffle(x1_, distributed=distributed)
             x0_ = self.backbone(x0_).flatten(start_dim=1)
             x0_ = self.projection_head(x0_)
 
             x1_ = self.backbone_momentum(x1_).flatten(start_dim=1)
             x1_ = self.projection_head_momentum(x1_)
-            x1_ = batch_unshuffle(x1_, shuffle)
+            x1_ = batch_unshuffle(x1_, shuffle, distributed=distributed)
             return x0_, x1_
 
         # We use a symmetric loss (model trains faster at little compute overhead)
@@ -218,8 +238,12 @@ class MocoModel(BenchmarkModule):
 
     def configure_optimizers(self):
         params = list(self.backbone.parameters()) + list(self.projection_head.parameters())
-        optim = torch.optim.SGD(params, lr=6e-2,
-                                momentum=0.9, weight_decay=5e-4)
+        optim = torch.optim.SGD(
+            params, 
+            lr=6e-2,
+            momentum=0.9, 
+            weight_decay=5e-4,
+        )
         scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optim, max_epochs)
         return [optim], [scheduler]
 
@@ -231,12 +255,12 @@ class SimCLRModel(BenchmarkModule):
         resnet = lightly.models.ResNetGenerator('resnet-18')
         self.backbone = nn.Sequential(
             *list(resnet.children())[:-1],
-            nn.AdaptiveAvgPool2d(1),
+            nn.AdaptiveAvgPool2d(1)
         )
         # create a simclr model based on ResNet
         self.resnet_simclr = \
             lightly.models.SimCLR(self.backbone, num_ftrs=512)
-        self.criterion = lightly.loss.NTXentLoss()
+        self.criterion = lightly.loss.NTXentLoss(gather_distributed=gather_distributed)
             
     def forward(self, x):
         self.resnet_simclr(x)
@@ -249,8 +273,12 @@ class SimCLRModel(BenchmarkModule):
         return loss
 
     def configure_optimizers(self):
-        optim = torch.optim.SGD(self.resnet_simclr.parameters(), lr=6e-2,
-                                momentum=0.9, weight_decay=5e-4)
+        optim = torch.optim.SGD(
+            self.resnet_simclr.parameters(), 
+            lr=6e-2,
+            momentum=0.9, 
+            weight_decay=5e-4
+        )
         scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optim, max_epochs)
         return [optim], [scheduler]
 
@@ -262,7 +290,7 @@ class SimSiamModel(BenchmarkModule):
         resnet = lightly.models.ResNetGenerator('resnet-18')
         self.backbone = nn.Sequential(
             *list(resnet.children())[:-1],
-            nn.AdaptiveAvgPool2d(1),
+            nn.AdaptiveAvgPool2d(1)
         )
         # create a simsiam model based on ResNet
         self.resnet_simsiam = \
@@ -295,8 +323,12 @@ class SimSiamModel(BenchmarkModule):
         return loss
 
     def configure_optimizers(self):
-        optim = torch.optim.SGD(self.resnet_simsiam.parameters(), lr=6e-2,
-                                momentum=0.9, weight_decay=5e-4)
+        optim = torch.optim.SGD(
+            self.resnet_simsiam.parameters(), 
+            lr=6e-2,
+            momentum=0.9, 
+            weight_decay=5e-4
+        )
         scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optim, max_epochs)
         return [optim], [scheduler]
 
@@ -304,10 +336,11 @@ class BarlowTwinsModel(BenchmarkModule):
     def __init__(self, dataloader_kNN, num_classes):
         super().__init__(dataloader_kNN, num_classes)
         # create a ResNet backbone and remove the classification head
-        resnet = torchvision.models.resnet18()
+        resnet = lightly.models.ResNetGenerator('resnet-18')
         last_conv_channels = list(resnet.children())[-1].in_features
         self.backbone = nn.Sequential(
             *list(resnet.children())[:-1],
+            nn.AdaptiveAvgPool2d(1)
         )
         # create a barlow twins model based on ResNet
         self.resnet_barlowtwins = \
@@ -332,7 +365,7 @@ class BarlowTwinsModel(BenchmarkModule):
                 None
             )
         ])
-        self.criterion = lightly.loss.BarlowTwinsLoss()
+        self.criterion = lightly.loss.BarlowTwinsLoss(gather_distributed=gather_distributed)
 
     def forward(self, x):
         self.resnet_barlowtwins(x)
@@ -345,8 +378,12 @@ class BarlowTwinsModel(BenchmarkModule):
         return loss
 
     def configure_optimizers(self):
-        optim = torch.optim.SGD(self.resnet_barlowtwins.parameters(), lr=6e-2,
-                                momentum=0.9, weight_decay=5e-4)
+        optim = torch.optim.SGD(
+            self.resnet_barlowtwins.parameters(), 
+            lr=6e-2,
+            momentum=0.9, 
+            weight_decay=5e-4
+        )
         scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optim, max_epochs)
         return [optim], [scheduler]
 
@@ -354,7 +391,7 @@ class BYOLModel(BenchmarkModule):
     def __init__(self, dataloader_kNN, num_classes):
         super().__init__(dataloader_kNN, num_classes)
         # create a ResNet backbone and remove the classification head
-        resnet = torchvision.models.resnet18()
+        resnet = lightly.models.ResNetGenerator('resnet-18')
         self.backbone = nn.Sequential(
             *list(resnet.children())[:-1],
             nn.AdaptiveAvgPool2d(1)
@@ -403,8 +440,12 @@ class BYOLModel(BenchmarkModule):
         params = list(self.backbone.parameters()) \
             + list(self.projection_head.parameters()) \
             + list(self.prediction_head.parameters())
-        optim = torch.optim.SGD(params, lr=6e-2,
-                                momentum=0.9, weight_decay=5e-4)
+        optim = torch.optim.SGD(
+            params, 
+            lr=6e-2,
+            momentum=0.9, 
+            weight_decay=5e-4,
+        )
         scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optim, max_epochs)
         return [optim], [scheduler]
 
@@ -412,7 +453,7 @@ class SwaVModel(BenchmarkModule):
     def __init__(self, dataloader_kNN, num_classes):
         super().__init__(dataloader_kNN, num_classes)
         # create a ResNet backbone and remove the classification head
-        resnet = torchvision.models.resnet18()
+        resnet = lightly.models.ResNetGenerator('resnet-18')
         self.backbone = nn.Sequential(
             *list(resnet.children())[:-1],
             nn.AdaptiveAvgPool2d(1)
@@ -421,7 +462,7 @@ class SwaVModel(BenchmarkModule):
         self.projection_head = SwaVProjectionHead(512, 512, 128)
         self.prototypes = SwaVPrototypes(128, 512) # use 512 prototypes
 
-        self.criterion = lightly.loss.SwaVLoss()
+        self.criterion = lightly.loss.SwaVLoss(sinkhorn_gather_distributed=gather_distributed)
 
     def forward(self, x):
         x = self.backbone(x).flatten(start_dim=1)
@@ -464,45 +505,59 @@ class SwaVModel(BenchmarkModule):
         scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optim, max_epochs)
         return [optim], [scheduler]
 
-model_names = ['MoCo_128', 'SimCLR_128', 'SimSiam_128', 'BarlowTwinsModel_128', 'BYOL_128', 'SwAV_128',
-               'MoCo_512', 'SimCLR_512', 'SimSiam_512', 'BarlowTwinsModel_512', 'BYOL_512', 'SwAV_512']
 models = [MocoModel, SimCLRModel, SimSiamModel, BarlowTwinsModel, BYOLModel, SwaVModel]
-bench_results = []
-gpu_memory_usage = []
+bench_results = dict()
 
 # loop through configurations and train models
 for batch_size in batch_sizes:
-    for model_name, BenchmarkModel in zip(model_names, models):
+    for BenchmarkModel in models:
         runs = []
+        model_name = BenchmarkModel.__name__
         for seed in range(n_runs):
             pl.seed_everything(seed)
             dataloader_train_ssl, dataloader_train_kNN, dataloader_test = get_data_loaders(batch_size)
             benchmark_model = BenchmarkModel(dataloader_train_kNN, classes)
 
-            logger = TensorBoardLogger('imagenette_runs', version=model_name)
+            logger = TensorBoardLogger('cifar10_runs', version=model_name)
 
-            trainer = pl.Trainer(max_epochs=max_epochs, 
-                                gpus=gpus,
-                                progress_bar_refresh_rate=100,
-                                distributed_backend=distributed_backend,
-                                default_root_dir=logs_root_dir)
+            trainer = pl.Trainer(
+                max_epochs=max_epochs, 
+                gpus=gpus,
+                default_root_dir=logs_root_dir,
+                strategy=distributed_backend,
+                sync_batchnorm=sync_batchnorm,
+            )
+            start = time.time()
             trainer.fit(
                 benchmark_model,
-                train_dataloader=dataloader_train_ssl,
+                train_dataloaders=dataloader_train_ssl,
                 val_dataloaders=dataloader_test
             )
-            gpu_memory_usage.append(torch.cuda.max_memory_allocated())
-            torch.cuda.reset_peak_memory_stats()
-            runs.append(benchmark_model.max_accuracy)
+            end = time.time()
+            run = {
+                'seed': seed,
+                'runtime': end - start,
+                'max_accuracy': benchmark_model.max_accuracy,
+                'gpu_memory_usage': torch.cuda.max_memory_allocated(),
+            }
+            runs.append(run)
 
             # delete model and trainer + free up cuda memory
             del benchmark_model
             del trainer
+            torch.cuda.reset_peak_memory_stats()
             torch.cuda.empty_cache()
-        bench_results.append(runs)
+        
+        bench_results[model_name] = runs
 
-for result, model, gpu_usage in zip(bench_results, model_names, gpu_memory_usage):
-    result_np = np.array(result)
-    mean = result_np.mean()
-    std = result_np.std()
-    print(f'{model}: {mean:.3f} +- {std:.3f}, GPU used: {gpu_usage / (1024.0**3):.1f} GByte', flush=True)
+for model, results in bench_results.items():
+    runtime = np.array([result['runtime'] for result in results])
+    accuracy = np.array([result['max_accuracy'] for result in results])
+    gpu_memory_usage = np.array([result['gpu_memory_usage'] for result in results])
+
+    print(
+        f'{model}: {accuracy.mean():.3f} +- {accuracy.std():.3f}'
+        f', GPU used: {gpu_memory_usage.max() / (1024.0**3):.1f} GByte'
+        f', Time: {runtime.mean() // 60} min',
+        flush=True
+    )
